@@ -419,3 +419,26 @@ def test_rect_stroke_zero():
               stroke-width="0" stroke="red" fill="none" />
       </svg>
     ''')
+
+
+@assert_no_logs
+def test_rect_width_height_zero():
+    assert_pixels('rect_fill', 9, 9, '''
+        _________
+        _________
+        _________
+        _________
+        _________
+        _________
+        _________
+        _________
+        _________
+    ''', '''
+        <style>
+            @page { size: 9px }
+            svg { display: block }
+        </style>
+        <svg width="0" height="0" xmlns="http://www.w3.org/2000/svg">
+            <rect x="2" y="2" width="5" height="5" fill="red" />
+        </svg>
+    ''')
