@@ -782,7 +782,7 @@ def test_tables_9():
 
 @assert_no_logs
 def test_tables_10():
-    assert_pixels('collapsed_border_tfoot', 22, 36, '''
+    assert_pixels('collapsed_border_tfoot', 22, 34, '''
         ______________________
         __RRRRRRRRRRRRRRRRRR__
         __R_____R____R_____R__
@@ -817,8 +817,6 @@ def test_tables_10():
         ______________________
         ______________________
         ______________________
-        ______________________
-        ______________________
     ''', '''
       <style>
         @page { size: 22px 17px; margin: 1px; background: #fff }
@@ -1081,3 +1079,122 @@ def test_tables_15():
         <tr><td colspan="3"></td></tr>
         <tfoot style="border: blue solid; border-width: 3px;
             "><td></td><td></td><td></td></tfoot>''')
+
+
+@assert_no_logs
+def test_tables_16():
+    assert_pixels('table_absolute', 20, 10, '''
+      ____________________
+      _RRRRRRRRRRR________
+      _R____R____R________
+      _R____R____R________
+      _R____R_RRRRRRRRRRR_
+      _RRRRRRRRRRR_R____R_
+      ________R____R____R_
+      ________R____R____R_
+      ________RRRRRRRRRRR_
+      ____________________
+    ''', '''
+      <style>
+        @page { size: 20px 10px; margin: 1px; background: #fff }
+        body { text-align: right; font-size: 0 }
+        table { position: absolute; width: 11px;
+                table-layout: fixed; border-collapse: collapse }
+        td { border: 1px red solid; width: 4px; height: 3px }
+      </style>
+      <table style="top: 0; left: 0">
+        <tr><td></td><td></td></tr>
+      <table style="bottom: 0; right: 0">
+        <tr><td></td><td></td></tr>''')
+
+
+@assert_no_logs
+def test_tables_17():
+    assert_pixels('table_split_collapse', 16, 20, '''
+      ________________
+      _RRRRRRRRRRRRRR_
+      _RRRRRRRRRRRRRR_
+      _RR____RR____RR_
+      _RR_BB_RR_BB_RR_
+      _RR_BB_RR_BB_RR_
+      _RR_BB_RR____RR_
+      _RR_BB_RR____RR_
+      _RR____RR____RR_
+      ________________
+      ________________
+      _RR_BB_RR____RR_
+      _RR_BB_RR____RR_
+      _RR_BB_RR____RR_
+      _RR_BB_RR____RR_
+      _RR____RR____RR_
+      _RRRRRRRRRRRRRR_
+      _RRRRRRRRRRRRRR_
+      ________________
+      ________________
+    ''', '''
+      <style>
+        @font-face {src: url(weasyprint.otf); font-family: weasyprint}
+        @page { size: 16px 10px; margin: 1px; background: #fff }
+        table { border-collapse: collapse; font-size: 2px; line-height: 1;
+                color: blue; font-family: weasyprint }
+        td { border: 2px red solid; padding: 1px; line-height: 1 }
+      </style>
+      <table><tr><td>a a a a</td><td>a</td></tr>''')
+
+
+@assert_no_logs
+def test_tables_18():
+    assert_pixels('table_split_separate', 12, 22, '''
+      ____________
+      _RRRRRRRRRR_
+      _R________R_
+      _R_RRRRRR_R_
+      _R_R____R_R_
+      _R_R_BB_R_R_
+      _R_R_BB_R_R_
+      _R_R_BB_R_R_
+      _R_R_BB_R_R_
+      _R_R____R_R_
+      ____________
+      ____________
+      _R_R_BB_R_R_
+      _R_R_BB_R_R_
+      _R_R_BB_R_R_
+      _R_R_BB_R_R_
+      _R_R____R_R_
+      _R_RRRRRR_R_
+      _R________R_
+      _RRRRRRRRRR_
+      ____________
+      ____________
+    ''', '''
+      <style>
+        @font-face {src: url(weasyprint.otf); font-family: weasyprint}
+        @page { size: 12px 11px; margin: 1px; background: #fff }
+        table { border: 1px red solid; border-spacing: 1px; font-size: 2px;
+                line-height: 1; color: blue; font-family: weasyprint }
+        td { border: 1px red solid; padding: 1px; line-height: 1; }
+      </style>
+      <table><tr><td>a a a a</td></tr>''')
+
+
+@assert_no_logs
+def test_tables_19():
+    # Regression test: https://github.com/Kozea/WeasyPrint/issues/1523
+    assert_pixels('table_split_crash', 2, 8, '''
+      RR
+      RR
+      RR
+      RR
+      RR
+      RR
+      RR
+      RR
+    ''', '''
+      <style>
+        @font-face {src: url(weasyprint.otf); font-family: weasyprint}
+        @page {size: 2px 4px}
+        table {border-collapse: collapse; color: red}
+        body {font-size: 2px; font-family: weasyprint; line-height: 1}
+      </style>
+      <table><tr><td>a a a a</td></tr></table>''')
