@@ -303,8 +303,13 @@ def test_command_line_render(tmpdir):
     html = b'<body><img src=pattern.png>'
     combined = b'<style>' + css + b'</style>' + html
     linked = b'<link rel=stylesheet href=style.css>' + html
+    not_optimized = b'<body>a<img src="not-optimized.jpg">'
+
+    tmpdir.chdir()
+    for name in ('pattern.png', 'not-optimized.jpg'):
+        pattern_bytes = Path(resource_filename(name)).read_bytes()
+        tmpdir.join(name).write_binary(pattern_bytes)
 
-    py.path.local(resource_filename('')).chdir()
     # Reference
     html_obj = FakeHTML(string=combined, base_url='dummy.html')
     pdf_bytes = html_obj.write_pdf()
@@ -312,15 +317,12 @@ def test_command_line_render(tmpdir):
         string=combined, base_url='dummy.html',
         media_type='screen').write_pdf()
 
-    tmpdir.chdir()
-    with open(resource_filename('pattern.png'), 'rb') as pattern_fd:
-        pattern_bytes = pattern_fd.read()
-    tmpdir.join('pattern.png').write_binary(pattern_bytes)
     tmpdir.join('no_css.html').write_binary(html)
     tmpdir.join('combined.html').write_binary(combined)
     tmpdir.join('combined-UTF-16BE.html').write_binary(
         combined.decode('ascii').encode('UTF-16BE'))
     tmpdir.join('linked.html').write_binary(linked)
+    tmpdir.join('not_optimized.html').write_binary(not_optimized)
     tmpdir.join('style.css').write_binary(css)
 
     _run('combined.html out2.pdf')
@@ -364,6 +366,21 @@ def test_command_line_render(tmpdir):
     assert tmpdir.join('out13.pdf').read_binary() == rotated_pdf_bytes
     assert tmpdir.join('out14.pdf').read_binary() == rotated_pdf_bytes
 
+    _run('not_optimized.html out15.pdf')
+    _run('not_optimized.html out16.pdf -O images')
+    _run('not_optimized.html out17.pdf -O fonts')
+    _run('not_optimized.html out18.pdf -O fonts -O images')
+    _run('not_optimized.html out19.pdf -O all')
+    _run('not_optimized.html out20.pdf -O none')
+    _run('not_optimized.html out21.pdf -O none -O all')
+    _run('not_optimized.html out22.pdf -O all -O none')
+    # TODO: test that equivalent CLI options give equivalent PDF sizes,
+    # unfortunately font optimization makes PDF generation not reproducible
+    assert (
+        len(tmpdir.join('out16.pdf').read_binary()) <
+        len(tmpdir.join('out15.pdf').read_binary()) <
+        len(tmpdir.join('out20.pdf').read_binary()))
+
     stdout = _run('combined.html -')
     assert stdout.count(b'attachment') == 0
     stdout = _run('combined.html -')
@@ -958,3 +975,12 @@ def test_http():
         assert HTML(root_url + '/deflate').etree_element.get('test') == 'ok'
         assert HTML(
             root_url + '/raw-deflate').etree_element.get('test') == 'ok'
+
+
+@assert_no_logs
+def test_page_copy_relative():
+    # Regression test for https://github.com/Kozea/WeasyPrint/issues/1473
+    document = FakeHTML(string='<div style="position: relative">a').render()
+    duplicated_pages = document.copy([*document.pages, *document.pages])
+    pngs = duplicated_pages.write_png(split_images=True)
+    assert pngs[0] == pngs[1]
