@@ -9,7 +9,7 @@
 import pytest
 from weasyprint.css import PageType, get_all_computed_styles
 from weasyprint.formatting_structure import boxes, build
-from weasyprint.layout.pages import set_page_type_computed_styles
+from weasyprint.layout.page import set_page_type_computed_styles
 
 from .testing_utils import (
     FakeHTML, assert_no_logs, assert_tree, capture_logs, parse, parse_all,
@@ -64,7 +64,7 @@ def test_inline_in_block_1():
                     ('div', 'Text', 'Hello, '),
                     ('em', 'Inline', [
                         ('em', 'Text', 'World')]),
-                    ('div', 'Text', '!\n')])]),
+                    ('div', 'Text', '! ')])]),
             ('p', 'Block', [
                 ('p', 'Line', [
                     ('p', 'Text', 'Lipsum.')])])])]
@@ -86,7 +86,7 @@ def test_inline_in_block_2():
                     ('div', 'Text', 'Hello, '),
                     ('em', 'Inline', [
                         ('em', 'Text', 'World')]),
-                    ('div', 'Text', '!\n')])])])]
+                    ('div', 'Text', '! ')])])])]
     box = parse(source)
     box = build.inline_in_block(box)
     assert_tree(box, expected)
@@ -151,8 +151,7 @@ def test_block_in_inline():
                             ('span', 'Block', [  # This block is "pulled up"
                                 ('span', 'Line', [
                                     ('span', 'Text', 'sit')])]),
-                            # No whitespace processing here.
-                            ('strong', 'Text', '\n      '),
+                            ('strong', 'Text', ' '),
                             ('span', 'Block', [  # This block is "pulled up"
                                 ('span', 'Line', [
                                     ('span', 'Text', 'amet,')])])]),
@@ -180,8 +179,7 @@ def test_block_in_inline():
                     ('p', 'Line', [
                         ('em', 'Inline', [
                             ('strong', 'Inline', [
-                                # Whitespace processing not done yet.
-                                ('strong', 'Text', '\n      ')])])])]),
+                                ('strong', 'Text', ' ')])])])]),
                 ('span', 'Block', [
                     ('span', 'Line', [
                         ('span', 'Text', 'amet,')])]),
@@ -1215,5 +1213,5 @@ def test_border_collapse_5():
 ))
 def test_display_none_root(html):
     box = parse_all(html)
-    assert box.style['display'] == 'block'
+    assert box.style['display'] == ('block', 'flow')
     assert not box.children
