@@ -14,22 +14,13 @@ from weasyprint import CSS, css, default_url_fetcher
 from weasyprint.css import (
     PageType, get_all_computed_styles, parse_page_selectors)
 from weasyprint.css.computed_values import strut_layout
-from weasyprint.layout.pages import set_page_type_computed_styles
+from weasyprint.layout.page import set_page_type_computed_styles
 from weasyprint.urls import path2url
 
 from .testing_utils import (
     BASE_URL, FakeHTML, assert_no_logs, capture_logs, resource_filename)
 
 
-@assert_no_logs
-def test_style_dict():
-    style = {'margin_left': 12, 'display': 'block'}
-    assert style['display'] == 'block'
-    assert style['margin_left'] == 12
-    with pytest.raises(KeyError):
-        style['position']
-
-
 @assert_no_logs
 def test_find_stylesheets():
     html = FakeHTML(resource_filename('doc1.html'))
