@@ -14,6 +14,8 @@ from codecs import BOM_UTF16_BE
 
 import pytest
 from weasyprint import Attachment
+from weasyprint.document import Document, DocumentMetadata
+from weasyprint.text.fonts import FontConfiguration
 from weasyprint.urls import path2url
 
 from .testing_utils import (
@@ -25,19 +27,6 @@ TOP = round(297 * 72 / 25.4, 6)
 RIGHT = round(210 * 72 / 25.4, 6)
 
 
-def assert_rect_almost_equal(rect, values):
-    """Test that PDF rect string equals given values.
-
-    We avoid rounding errors by allowing a delta of 1, as both WeasyPrint and
-    pydyf round coordinates in unpredictable ways.
-
-    """
-    if isinstance(rect, bytes):
-        rect = rect.decode('ascii')
-    for a, b in zip(rect.strip(' []').split(), values):
-        assert abs(int(a) - b) <= 1
-
-
 @assert_no_logs
 @pytest.mark.parametrize('zoom', (1, 1.5, 0.5))
 def test_page_size_zoom(zoom):
@@ -279,6 +268,33 @@ def test_bookmarks_12():
     assert re.findall(b'/Title \\((.*)\\)', pdf) == [b'a']
 
 
+@assert_no_logs
+def test_bookmarks_13():
+    pdf = FakeHTML(string='''
+      <div style="bookmark-level:1; bookmark-label:contents;
+                  text-transform:uppercase">a</div>
+    ''').write_pdf()
+    # a
+    counts = re.findall(b'/Count ([0-9-]*)', pdf)
+    outlines = counts.pop()
+    assert outlines == b'1'
+    assert re.findall(b'/Title \\((.*)\\)', pdf) == [b'a']
+
+
+@assert_no_logs
+def test_bookmarks_14():
+    pdf = FakeHTML(string='''
+      <h1>a</h1>
+      <h1> b c d </h1>
+      <h1> e
+             f </h1>
+      <h1> g <span> h </span> i </h1>
+    ''').write_pdf()
+    assert re.findall(b'/Count ([0-9-]*)', pdf)[-1] == b'4'
+    assert re.findall(b'/Title \\((.*)\\)', pdf) == [
+        b'a', b'b c d', b'e f', b'g h i']
+
+
 @assert_no_logs
 def test_links_none():
     pdf = FakeHTML(string='<body>').write_pdf()
@@ -424,13 +440,30 @@ def test_relative_links_anchors():
     assert [float(number) for number in rect.split()] == [0, TOP, RIGHT, TOP]
 
 
+@assert_no_logs
+def test_relative_links_different_base():
+    pdf = FakeHTML(
+        string='<a href="/test/lipsum"></a>a',
+        base_url='http://weasyprint.org/foo/bar/').write_pdf()
+    assert b'http://weasyprint.org/test/lipsum' in pdf
+
+
+@assert_no_logs
+def test_relative_links_same_base():
+    pdf = FakeHTML(
+        string='<a id="test" href="/foo/bar/#test"></a>a',
+        base_url='http://weasyprint.org/foo/bar/').write_pdf()
+    assert b'/Dest (test)' in pdf
+
+
 @assert_no_logs
 def test_missing_links():
     with capture_logs() as logs:
         pdf = FakeHTML(string='''
           <style> a { display: block; height: 15pt } </style>
           <a href="#lipsum"></a>
-          <a href="#missing" id="lipsum"></a>a
+          <a href="#missing" id="lipsum"></a>
+          <a href=""></a>a
         ''', base_url=None).write_pdf()
     assert b'/Dest (lipsum)' in pdf
     assert len(logs) == 1
@@ -474,6 +507,21 @@ def test_embed_image_once():
         ''').write_pdf().count(b'/Filter /DCTDecode') == 1
 
 
+@assert_no_logs
+def test_embed_images_from_pages():
+    page1, = FakeHTML(
+        base_url=resource_filename('dummy.html'),
+        string='<img src="blue.jpg">').render().pages
+    page2, = FakeHTML(
+        base_url=resource_filename('dummy.html'),
+        string='<img src="not-optimized.jpg">').render().pages
+    document = Document(
+        (page1, page2), metadata=DocumentMetadata(),
+        font_config=FontConfiguration(), url_fetcher=None,
+        optimize_size=()).write_pdf()
+    assert document.count(b'/Filter /DCTDecode') == 2
+
+
 @assert_no_logs
 def test_document_info():
     pdf = FakeHTML(string='''
