@@ -28,10 +28,10 @@ def test_line_content():
     for width, remaining in [(100, 'text for test'),
                              (45, 'is a text for test')]:
         text = 'This is a text for test'
-        _, length, resume_at, _, _, _ = make_text(
+        _, length, resume_index, _, _, _ = make_text(
             text, width, font_family=SANS_FONTS.split(','), font_size=19)
-        assert text[resume_at:] == remaining
-        assert length + 1 == resume_at  # +1 is for the removed trailing space
+        assert text[resume_index:] == remaining
+        assert length + 1 == resume_index  # +1 for the removed trailing space
 
 
 @assert_no_logs
@@ -46,16 +46,16 @@ def test_line_breaking():
     string = 'Thïs is a text for test'
 
     # These two tests do not really rely on installed fonts
-    _, _, resume_at, _, _, _ = make_text(string, 90, font_size=1)
-    assert resume_at is None
+    _, _, resume_index, _, _, _ = make_text(string, 90, font_size=1)
+    assert resume_index is None
 
-    _, _, resume_at, _, _, _ = make_text(string, 90, font_size=100)
-    assert string.encode('utf-8')[resume_at:].decode('utf-8') == (
+    _, _, resume_index, _, _, _ = make_text(string, 90, font_size=100)
+    assert string.encode('utf-8')[resume_index:].decode('utf-8') == (
         'is a text for test')
 
-    _, _, resume_at, _, _, _ = make_text(
+    _, _, resume_index, _, _, _ = make_text(
         string, 100, font_family=SANS_FONTS.split(','), font_size=19)
-    assert string.encode('utf-8')[resume_at:].decode('utf-8') == (
+    assert string.encode('utf-8')[resume_index:].decode('utf-8') == (
         'text for test')
 
 
@@ -64,11 +64,11 @@ def test_line_breaking_rtl():
     string = 'لوريم ايبسوم دولا'
 
     # These two tests do not really rely on installed fonts
-    _, _, resume_at, _, _, _ = make_text(string, 90, font_size=1)
-    assert resume_at is None
+    _, _, resume_index, _, _, _ = make_text(string, 90, font_size=1)
+    assert resume_index is None
 
-    _, _, resume_at, _, _, _ = make_text(string, 90, font_size=100)
-    assert string.encode('utf-8')[resume_at:].decode('utf-8') == (
+    _, _, resume_index, _, _, _ = make_text(string, 90, font_size=100)
+    assert string.encode('utf-8')[resume_index:].decode('utf-8') == (
         'ايبسوم دولا')
 
 
@@ -99,6 +99,23 @@ def test_text_font_size_zero():
     assert paragraph.height == 0
 
 
+@assert_no_logs
+def test_text_font_size_very_small():
+    # Test regression: https://github.com/Kozea/WeasyPrint/issues/1499
+    page, = render_pages('''
+      <style>
+        p { font-size: 0.00000001px }
+      </style>
+      <p>test font size zero</p>
+    ''')
+    html, = page.children
+    body, = html.children
+    paragraph, = body.children
+    line, = paragraph.children
+    assert line.height < 0.001
+    assert paragraph.height < 0.001
+
+
 @assert_no_logs
 def test_text_spaced_inlines():
     page, = render_pages('''
@@ -244,6 +261,86 @@ def test_text_align_justify():
     assert image_5.position_x == 0
 
 
+@assert_no_logs
+def test_text_align_justify_all():
+    page, = render_pages('''
+      <style>
+        @page { size: 300px 1000px }
+        body { text-align: justify-all }
+      </style>
+      <p><img src="pattern.png" style="width: 40px">
+        <strong>
+          <img src="pattern.png" style="width: 60px">
+          <img src="pattern.png" style="width: 10px">
+          <img src="pattern.png" style="width: 100px"
+        ></strong><img src="pattern.png" style="width: 200px">
+        <img src="pattern.png" style="width: 10px">''')
+    html, = page.children
+    body, = html.children
+    paragraph, = body.children
+    line_1, line_2 = paragraph.children
+    image_1, space_1, strong = line_1.children
+    image_2, space_2, image_3, space_3, image_4 = strong.children
+    image_5, space_4, image_6 = line_2.children
+    assert space_1.text == ' '
+    assert space_2.text == ' '
+    assert space_3.text == ' '
+    assert space_4.text == ' '
+
+    assert image_1.position_x == 0
+    assert space_1.position_x == 40
+    assert strong.position_x == 70
+    assert image_2.position_x == 70
+    assert space_2.position_x == 130
+    assert image_3.position_x == 160
+    assert space_3.position_x == 170
+    assert image_4.position_x == 200
+    assert strong.width == 230
+
+    assert image_5.position_x == 0
+    assert space_4.position_x == 200
+    assert image_6.position_x == 290
+
+
+@assert_no_logs
+def test_text_align_all_last():
+    page, = render_pages('''
+      <style>
+        @page { size: 300px 1000px }
+        body { text-align-all: justify; text-align-last: right }
+      </style>
+      <p><img src="pattern.png" style="width: 40px">
+        <strong>
+          <img src="pattern.png" style="width: 60px">
+          <img src="pattern.png" style="width: 10px">
+          <img src="pattern.png" style="width: 100px"
+        ></strong><img src="pattern.png" style="width: 200px"
+        ><img src="pattern.png" style="width: 10px">''')
+    html, = page.children
+    body, = html.children
+    paragraph, = body.children
+    line_1, line_2 = paragraph.children
+    image_1, space_1, strong = line_1.children
+    image_2, space_2, image_3, space_3, image_4 = strong.children
+    image_5, image_6 = line_2.children
+    assert space_1.text == ' '
+    assert space_2.text == ' '
+    assert space_3.text == ' '
+
+    assert image_1.position_x == 0
+    assert space_1.position_x == 40
+    assert strong.position_x == 70
+    assert image_2.position_x == 70
+    assert space_2.position_x == 130
+    assert image_3.position_x == 160
+    assert space_3.position_x == 170
+    assert image_4.position_x == 200
+    assert strong.width == 230
+
+    assert image_5.position_x == 90
+    assert image_6.position_x == 290
+
+
 @assert_no_logs
 def test_text_align_not_enough_space():
     page, = render_pages('''
@@ -794,6 +891,7 @@ def test_hyphenate_limit_chars_punctuation(css):
 
 @assert_no_logs
 @pytest.mark.parametrize('wrap, text, test, full_text', (
+    ('anywhere', 'aaaaaaaa', lambda a: a > 1, 'aaaaaaaa'),
     ('break-word', 'aaaaaaaa', lambda a: a > 1, 'aaaaaaaa'),
     ('normal', 'aaaaaaaa', lambda a: a == 1, 'aaaaaaaa'),
     ('break-word', 'hyphenations', lambda a: a > 3,
@@ -805,8 +903,8 @@ def test_overflow_wrap(wrap, text, test, full_text):
     page, = render_pages('''
       <style>
         @font-face {src: url(weasyprint.otf); font-family: weasyprint}
-        body {width: 80px; overflow: hidden; font-family: weasyprint; }
-        span {overflow-wrap: %s; white-space: normal; }
+        body {width: 80px; overflow: hidden; font-family: weasyprint}
+        span {overflow-wrap: %s}
       </style>
       <body style="hyphens: auto;" lang="en">
         <span>%s
@@ -823,6 +921,58 @@ def test_overflow_wrap(wrap, text, test, full_text):
     assert full_text == lines_full_text
 
 
+@assert_no_logs
+@pytest.mark.parametrize('wrap, text, body_width, expected_width', (
+    ('anywhere', 'aaaaaa', 10, 20),
+    ('anywhere', 'aaaaaa', 40, 40),
+    ('break-word', 'aaaaaa', 40, 120),
+    ('normal', 'aaaaaa', 40, 120),
+))
+def test_overflow_wrap_2(wrap, text, body_width, expected_width):
+    page, = render_pages('''
+      <style>
+        @font-face {src: url(weasyprint.otf); font-family: weasyprint}
+        body {width: %dpx; font-family: weasyprint; font-size: 20px}
+        table {overflow-wrap: %s}
+      </style>
+      <table><tr><td>%s''' % (body_width, wrap, text))
+    html, = page.children
+    body, = html.children
+    table_wrapper, = body.children
+    table, = table_wrapper.children
+    row_group, = table.children
+    tr, = row_group.children
+    td, = tr.children
+    assert td.width == expected_width
+
+
+@assert_no_logs
+@pytest.mark.parametrize('wrap, text, body_width, expected_width', (
+    # TODO: broken with Pango 1.50,
+    # see https://gitlab.gnome.org/GNOME/pango/-/issues/646
+    # ('anywhere', 'aaaaaa', 10, 20),
+    ('anywhere', 'aaaaaa', 40, 40),
+    ('break-word', 'aaaaaa', 40, 120),
+    ('normal', 'abcdef', 40, 120),
+))
+def test_overflow_wrap_trailing_space(wrap, text, body_width, expected_width):
+    page, = render_pages('''
+      <style>
+        @font-face {src: url(weasyprint.otf); font-family: weasyprint}
+        body {width: %dpx; font-family: weasyprint; font-size: 20px}
+        table {overflow-wrap: %s}
+      </style>
+      <table><tr><td>%s ''' % (body_width, wrap, text))
+    html, = page.children
+    body, = html.children
+    table_wrapper, = body.children
+    table, = table_wrapper.children
+    row_group, = table.children
+    tr, = row_group.children
+    td, = tr.children
+    assert td.width == expected_width
+
+
 def white_space_lines(width, space):
     page, = render_pages('''
       <style>
