@@ -32,7 +32,8 @@ from .validation.descriptors import preprocess_descriptors
 
 # Reject anything not in here:
 PSEUDO_ELEMENTS = (
-    None, 'before', 'after', 'marker', 'first-line', 'first-letter')
+    None, 'before', 'after', 'marker', 'first-line', 'first-letter',
+    'footnote-call', 'footnote-marker')
 
 
 PageType = namedtuple('PageType', ['side', 'blank', 'first', 'index', 'name'])
@@ -122,17 +123,17 @@ class StyleFor:
         style = self._computed_styles.get((element, pseudo_type))
 
         if style:
-            if 'table' in style['display']:
-                if (style['display'] in ('table', 'inline-table') and
-                        style['border_collapse'] == 'collapse'):
-                    # Padding do not apply
-                    for side in ['top', 'bottom', 'left', 'right']:
-                        style[f'padding_{side}'] = computed_values.ZERO_PIXELS
-                if (style['display'].startswith('table-') and
-                        style['display'] != 'table-caption'):
-                    # Margins do not apply
-                    for side in ['top', 'bottom', 'left', 'right']:
-                        style[f'margin_{side}'] = computed_values.ZERO_PIXELS
+            if ('table' in style['display'] and
+                    style['border_collapse'] == 'collapse'):
+                # Padding do not apply
+                for side in ['top', 'bottom', 'left', 'right']:
+                    style[f'padding_{side}'] = computed_values.ZERO_PIXELS
+            if (len(style['display']) == 1 and
+                    style['display'][0].startswith('table-') and
+                    style['display'][0] != 'table-caption'):
+                # Margins do not apply
+                for side in ['top', 'bottom', 'left', 'right']:
+                    style[f'margin_{side}'] = computed_values.ZERO_PIXELS
 
         return style
 
@@ -171,7 +172,7 @@ class StyleFor:
             for pseudo in (None, 'before', 'after'):
                 pseudo_style = cascaded_styles.get((element, pseudo), {})
                 if 'display' in pseudo_style:
-                    if pseudo_style['display'][0] == 'list-item':
+                    if 'list-item' in pseudo_style['display'][0]:
                         break
             else:
                 if (element, 'marker') in cascaded_styles:
@@ -238,7 +239,7 @@ def find_stylesheets(wrapper_element, device_media_type, url_fetcher, base_url,
     The output order is the same as the source order.
 
     """
-    from ..html import element_has_link_type  # Work around circular imports.
+    from ..html import element_has_link_type
 
     for wrapper in wrapper_element.query_all('style', 'link'):
         element = wrapper.etree_element
