@@ -21,7 +21,7 @@ INITIAL_VALUES = {
     'clip': (),  # computed value for 'auto'
     'color': parse_color('black'),  # chosen by the user agent
     'direction': 'ltr',
-    'display': 'inline',
+    'display': ('inline', 'flow'),
     'empty_cells': 'show',
     'float': 'none',
     'height': 'auto',
@@ -120,6 +120,8 @@ INITIAL_VALUES = {
     'bookmark_level': 'none',
     'bookmark_state': 'open',
     'content': 'normal',
+    'footnote_display': 'block',
+    'footnote_policy': 'auto',
     'quotes': list('“”‘’'),  # chosen by the user agent
     'string_set': 'none',
 
@@ -147,7 +149,8 @@ INITIAL_VALUES = {
     'hyphens': 'manual',
     'letter_spacing': 'normal',
     'tab_size': 8,
-    'text_align': 'start',
+    'text_align_all': 'start',
+    'text_align_last': 'auto',
     'text_indent': Dimension(0, 'px'),
     'text_transform': 'none',
     'white_space': 'normal',
@@ -257,7 +260,8 @@ INHERITED = {
     'overflow_wrap',
     'quotes',
     'tab_size',
-    'text_align',
+    'text_align_all',
+    'text_align_last',
     'text_decoration_line',
     'text_decoration_color',
     'text_decoration_style',
