@@ -9,12 +9,16 @@
 import contextlib
 import operator
 from colorsys import hsv_to_rgb, rgb_to_hsv
+from io import BytesIO
 from math import ceil, floor, pi, sqrt, tan
+from xml.etree import ElementTree
+
+from PIL import Image
 
 from .formatting_structure import boxes
-from .images import SVGImage
+from .images import RasterImage, SVGImage
 from .layout import replaced
-from .layout.backgrounds import BackgroundLayer
+from .layout.background import BackgroundLayer
 from .stacking import StackingContext
 from .text.ffi import ffi, harfbuzz, pango, units_from_double, units_to_double
 from .text.line_break import get_last_word_end
@@ -277,8 +281,7 @@ def draw_background(stream, bg, clip_box=True, bleed=None, marks=()):
             svg = f'''
               <svg height="{height}" width="{width}"
                    fill="transparent" stroke="black" stroke-width="1"
-                   xmlns="http://www.w3.org/2000/svg"
-                   xmlns:xlink="http://www.w3.org/1999/xlink">
+                   xmlns="http://www.w3.org/2000/svg">
             '''
             if 'crop' in marks:
                 svg += f'''
@@ -323,7 +326,8 @@ def draw_background(stream, bg, clip_box=True, bleed=None, marks=()):
                     translate({width},0) scale(0.5) translate(0,{height})" />
                 '''
             svg += '</svg>'
-            image = SVGImage(svg, None, None, stream)
+            tree = ElementTree.fromstring(svg)
+            image = SVGImage(tree, None, None, stream)
             # Painting area is the PDF media box
             size = (width, height)
             position = (x, y)
@@ -853,15 +857,23 @@ def draw_collapsed_borders(stream, table):
         if width == 0 or color.alpha == 0:
             return
         pos_x = column_positions[x]
-        pos_y1 = row_positions[y] - half_max_width(horizontal_borders, [
-            (y, x - 1), (y, x)], vertical=False)
-        pos_y2 = row_positions[y + 1] + half_max_width(horizontal_borders, [
-            (y + 1, x - 1), (y + 1, x)], vertical=False)
+        pos_y1 = row_positions[y]
+        if y != 0 or not table.skip_cell_border_top:
+            pos_y1 -= half_max_width(horizontal_borders, [
+                (y, x - 1), (y, x)], vertical=False)
+        pos_y2 = row_positions[y + 1]
+        if y != grid_height - 1 or not table.skip_cell_border_bottom:
+            pos_y2 += half_max_width(horizontal_borders, [
+                (y + 1, x - 1), (y + 1, x)], vertical=False)
         segments.append((
             score, style, width, color, 'left',
             (pos_x - width / 2, pos_y1, 0, pos_y2 - pos_y1)))
 
     def add_horizontal(x, y):
+        if y == 0 and table.skip_cell_border_top:
+            return
+        if y == grid_height and table.skip_cell_border_bottom:
+            return
         yy = row_number(y, horizontal=True)
         score, (style, width, color) = horizontal_borders[yy][x]
         if width == 0 or color.alpha == 0:
@@ -982,9 +994,12 @@ def draw_text(stream, textbox, offset_x, text_overflow, block_ellipsis):
 
     textbox.pango_layout.reactivate(textbox.style)
     stream.begin_text()
-    draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y)
+    emojis = draw_first_line(
+        stream, textbox, text_overflow, block_ellipsis, x, y)
     stream.end_text()
 
+    draw_emojis(stream, textbox.style['font_size'], x, y, emojis)
+
     # Draw text decoration
     values = textbox.style['text_decoration_line']
     color = textbox.style['text_decoration_color']
@@ -1010,6 +1025,15 @@ def draw_text(stream, textbox, offset_x, text_overflow, block_ellipsis):
     textbox.pango_layout.deactivate()
 
 
+def draw_emojis(stream, font_size, x, y, emojis):
+    for emoji in emojis:
+        for image, font, a, d, e, f in emojis:
+            stream.push_state()
+            stream.transform(a=a, d=d, e=x + e * font_size, f=y + f)
+            image.draw(stream, font_size, font_size, None)
+            stream.pop_state()
+
+
 def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
     """Draw the given ``textbox`` line to the document ``stream``."""
     pango.pango_layout_set_single_paragraph_mode(
@@ -1066,6 +1090,8 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
     last_font = None
     string = ''
     fonts = stream.get_fonts()
+    x_advance = 0
+    emojis = []
     for run in runs:
         # Pango objects
         glyph_item = ffi.cast('PangoGlyphItem *', run.data)
@@ -1083,7 +1109,9 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
         if font_hash in fonts:
             font = fonts[font_hash]
         else:
-            hb_blob = harfbuzz.hb_face_reference_blob(hb_face)
+            hb_blob = ffi.gc(
+                harfbuzz.hb_face_reference_blob(hb_face),
+                harfbuzz.hb_blob_destroy)
             hb_data = harfbuzz.hb_blob_get_data(hb_blob, stream.length)
             file_content = ffi.unpack(hb_data, int(stream.length[0]))
             index = harfbuzz.hb_face_get_index(hb_face)
@@ -1103,13 +1131,16 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
         string += '<'
         for i in range(num_glyphs):
             glyph_info = glyphs[i]
+            glyph = glyph_info.glyph
             width = glyph_info.geometry.width
+            if glyph == pango.PANGO_GLYPH_EMPTY:
+                string += f'>{-width / font_size}<'
+                continue
             utf8_position = utf8_positions[i]
 
             offset = glyph_info.geometry.x_offset / font_size
             if offset:
                 string += f'>{-offset}<'
-            glyph = glyph_info.glyph
             string += f'{glyph:04x}'
 
             # Ink bounding box and logical widths in font
@@ -1142,11 +1173,45 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
                 string += f'>{kerning}<'
 
             # Mapping between glyphs and characters
-            if glyph not in font.cmap and glyph != pango.PANGO_GLYPH_EMPTY:
+            if glyph not in font.cmap:
                 utf8_slice = slice(previous_utf8_position, utf8_position)
                 font.cmap[glyph] = utf8_text[utf8_slice].decode('utf-8')
             previous_utf8_position = utf8_position
 
+            if font.svg:
+                hb_blob = ffi.gc(
+                    harfbuzz.hb_ot_color_glyph_reference_svg(hb_face, glyph),
+                    harfbuzz.hb_blob_destroy)
+                hb_data = harfbuzz.hb_blob_get_data(hb_blob, stream.length)
+                if hb_data != ffi.NULL:
+                    svg_data = ffi.unpack(hb_data, int(stream.length[0]))
+                    tree = ElementTree.fromstring(svg_data)
+                    image = SVGImage(tree, None, None, stream)
+                    a = d = font.widths[glyph] / 1000 / font.upem * font_size
+                    emojis.append([image, font, a, d, x_advance, 0])
+            elif font.png:
+                hb_blob = ffi.gc(
+                    harfbuzz.hb_ot_color_glyph_reference_png(hb_font, glyph),
+                    harfbuzz.hb_blob_destroy)
+                hb_data = harfbuzz.hb_blob_get_data(hb_blob, stream.length)
+                if hb_data != ffi.NULL:
+                    png_data = ffi.unpack(hb_data, int(stream.length[0]))
+                    pillow_image = Image.open(BytesIO(png_data))
+                    image_id = f'{font.hash}{glyph}'
+                    image = RasterImage(
+                        pillow_image, image_id, optimize_size=())
+                    d = font.widths[glyph] / 1000
+                    a = pillow_image.width / pillow_image.height * d
+                    pango.pango_font_get_glyph_extents(
+                        pango_font, glyph, stream.ink_rect,
+                        stream.logical_rect)
+                    f = units_to_double(
+                        (-stream.logical_rect.y - stream.logical_rect.height))
+                    f = f / font_size - font_size
+                    emojis.append([image, font, a, d, x_advance, f])
+
+            x_advance += (font.widths[glyph] + offset) / 1000
+
         # Close the last glyphs list, remove if empty
         if string[-1] == '<':
             string = string[:-1]
@@ -1156,6 +1221,8 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
     # Draw text
     stream.show_text(string)
 
+    return emojis
+
 
 def draw_wave(stream, x, y, width, offset_x, radius):
     up = 1
