@@ -83,6 +83,7 @@ class Box:
     bookmark_label = None
     string_set = None
     download_name = None
+    footnote = None
 
     # Default, overriden on some subclasses
     def all_children(self):
@@ -273,7 +274,11 @@ class Box:
 
     def is_floated(self):
         """Return whether this box is floated."""
-        return self.style['float'] != 'none'
+        return self.style['float'] in ('left', 'right')
+
+    def is_footnote(self):
+        """Return whether this box is a footnote."""
+        return self.style['float'] == 'footnote'
 
     def is_absolutely_positioned(self):
         """Return whether this box is in the absolute positioning scheme."""
@@ -287,7 +292,7 @@ class Box:
         """Return whether this box is in normal flow."""
         return not (
             self.is_floated() or self.is_absolutely_positioned() or
-            self.is_running())
+            self.is_running() or self.is_footnote())
 
     # Start and end page values for named pages
 
@@ -468,24 +473,9 @@ class TextBox(InlineLevelBox):
     """
     justification_spacing = 0
 
-    # http://stackoverflow.com/questions/16317534/
-    ascii_to_wide = dict((i, chr(i + 0xfee0)) for i in range(0x21, 0x7f))
-    ascii_to_wide.update({0x20: '\u3000', 0x2D: '\u2212'})
-
     def __init__(self, element_tag, style, element, text):
         assert text
         super().__init__(element_tag, style, element)
-        text_transform = style['text_transform']
-        if text_transform != 'none':
-            text = {
-                'uppercase': lambda t: t.upper(),
-                'lowercase': lambda t: t.lower(),
-                # Python’s unicode.captitalize is not the same.
-                'capitalize': lambda t: t.title(),
-                'full-width': lambda t: t.translate(self.ascii_to_wide),
-            }[text_transform](text)
-        if style['hyphens'] == 'none':
-            text = text.replace('\u00AD', '')  # U+00AD SOFT HYPHEN (SHY)
         self.text = text
 
     def copy_with_text(self, text):
@@ -688,6 +678,18 @@ class MarginBox(BlockContainerBox):
         return f'<{type(self).__name__} {self.at_keyword}>'
 
 
+class FootnoteAreaBox(BlockBox):
+    """Box displaying footnotes, as defined in GCPM."""
+    def __init__(self, page, style):
+        self.page = page
+        # Footnote area boxes are not linked to any element.
+        super().__init__(
+            element_tag=None, style=style, element=None, children=[])
+
+    def __repr__(self):
+        return f'<{type(self).__name__} @footnote>'
+
+
 class FlexContainerBox(ParentBox):
     """A box that contains only flex-items."""
 
