@@ -23,31 +23,49 @@ from . import boxes
 
 # Maps values of the ``display`` CSS property to box types.
 BOX_TYPE_FROM_DISPLAY = {
-    'block': boxes.BlockBox,
-    'list-item': boxes.BlockBox,
-    'inline': boxes.InlineBox,
-    'inline-block': boxes.InlineBlockBox,
-    'table': boxes.TableBox,
-    'inline-table': boxes.InlineTableBox,
-    'table-row': boxes.TableRowBox,
-    'table-row-group': boxes.TableRowGroupBox,
-    'table-header-group': boxes.TableRowGroupBox,
-    'table-footer-group': boxes.TableRowGroupBox,
-    'table-column': boxes.TableColumnBox,
-    'table-column-group': boxes.TableColumnGroupBox,
-    'table-cell': boxes.TableCellBox,
-    'table-caption': boxes.TableCaptionBox,
-    'flex': boxes.FlexBox,
-    'inline-flex': boxes.InlineFlexBox,
+    ('block', 'flow'): boxes.BlockBox,
+    ('inline', 'flow'): boxes.InlineBox,
+
+    ('block', 'flow-root'): boxes.BlockBox,
+    ('inline', 'flow-root'): boxes.InlineBlockBox,
+
+    ('block', 'table'): boxes.TableBox,
+    ('inline', 'table'): boxes.InlineTableBox,
+
+    ('block', 'flex'): boxes.FlexBox,
+    ('inline', 'flex'): boxes.InlineFlexBox,
+
+    ('table-row',): boxes.TableRowBox,
+    ('table-row-group',): boxes.TableRowGroupBox,
+    ('table-header-group',): boxes.TableRowGroupBox,
+    ('table-footer-group',): boxes.TableRowGroupBox,
+    ('table-column',): boxes.TableColumnBox,
+    ('table-column-group',): boxes.TableColumnGroupBox,
+    ('table-cell',): boxes.TableCellBox,
+    ('table-caption',): boxes.TableCaptionBox,
 }
 
+# http://stackoverflow.com/questions/16317534/
+ASCII_TO_WIDE = {i: chr(i + 0xfee0) for i in range(0x21, 0x7f)}
+ASCII_TO_WIDE.update({0x20: '\u3000', 0x2D: '\u2212'})
+
+
+def create_anonymous_boxes(box):
+    """Create anonymous boxes in box descendants according to layout rules."""
+    box = anonymous_table_boxes(box)
+    box = flex_boxes(box)
+    box = inline_in_block(box)
+    box = block_in_inline(box)
+    return box
+
 
 def build_formatting_structure(element_tree, style_for, get_image_from_uri,
-                               base_url, target_collector, counter_style):
+                               base_url, target_collector, counter_style,
+                               footnotes):
     """Build a formatting structure (box tree) from an element tree."""
     box_list = element_to_box(
         element_tree, style_for, get_image_from_uri, base_url,
-        target_collector, counter_style)
+        target_collector, counter_style, footnotes)
     if box_list:
         box, = box_list
     else:
@@ -56,35 +74,31 @@ def build_formatting_structure(element_tree, style_for, get_image_from_uri,
             style = style_for(element, pseudo_type)
             if style is not None:
                 if element == element_tree:
-                    style['display'] = 'block'
+                    style['display'] = ('block', 'flow')
                 else:
-                    style['display'] = 'none'
+                    style['display'] = ('none',)
             return style
         box, = element_to_box(
             element_tree, root_style_for, get_image_from_uri, base_url,
-            target_collector, counter_style)
+            target_collector, counter_style, footnotes)
 
     target_collector.check_pending_targets()
 
     box.is_for_root_element = True
-    # If this is changed, maybe update weasy.layout.pages.make_margin_boxes()
-    process_whitespace(box)
-    box = anonymous_table_boxes(box)
-    box = flex_boxes(box)
-    box = inline_in_block(box)
-    box = block_in_inline(box)
+    # If this is changed, maybe update weasy.layout.page.make_margin_boxes()
+    box = create_anonymous_boxes(box)
     box = set_viewport_overflow(box)
     return box
 
 
 def make_box(element_tag, style, content, element):
-    box = BOX_TYPE_FROM_DISPLAY[style['display']](
+    box = BOX_TYPE_FROM_DISPLAY[style['display'][:2]](
         element_tag, style, element, content)
     return box
 
 
 def element_to_box(element, style_for, get_image_from_uri, base_url,
-                   target_collector, counter_style, state=None):
+                   target_collector, counter_style, footnotes, state=None):
     """Convert an element and its children into a box with children.
 
     Return a list of boxes. Most of the time the list will have one item but
@@ -117,9 +131,16 @@ def element_to_box(element, style_for, get_image_from_uri, base_url,
     # TODO: should be the used value. When does the used value for `display`
     # differ from the computer value?
     display = style['display']
-    if display == 'none':
+    if display == ('none',):
         return []
 
+    if style['float'] == 'footnote':
+        if style['footnote_display'] == 'block':
+            style['display'] = ('block', 'flow')
+        else:
+            # TODO: handle compact footnotes
+            style['display'] = ('inline', 'flow')
+
     box = make_box(element.tag, style, [], element)
 
     if state is None:
@@ -127,10 +148,11 @@ def element_to_box(element, style_for, get_image_from_uri, base_url,
         state = (
             # Shared mutable objects:
             [0],  # quote_depth: single integer
-            {},  # counter_values: name -> stacked/scoped values
-            [set()]  # counter_scopes: element tree depths -> counter names
+            # TODO: define the footnote counter where it can be updated by page
+            {'footnote': [0]},  # counter_values: name -> stacked/scoped values
+            [{'footnote'}]  # counter_scopes: element depths -> counter names
         )
-    _quote_depth, counter_values, counter_scopes = state
+    quote_depth, counter_values, counter_scopes = state
 
     update_counters(state, style)
 
@@ -144,7 +166,7 @@ def element_to_box(element, style_for, get_image_from_uri, base_url,
     box.first_line_style = style_for(element, 'first-line')
 
     marker_boxes = []
-    if style['display'] == 'list-item':
+    if 'list-item' in style['display']:
         marker_boxes = list(marker_to_box(
             element, state, style, style_for, get_image_from_uri,
             target_collector, counter_style))
@@ -165,9 +187,24 @@ def element_to_box(element, style_for, get_image_from_uri, base_url,
         children.append(boxes.TextBox.anonymous_from(box, text))
 
     for child_element in element:
-        children.extend(element_to_box(
+        child_boxes = element_to_box(
             child_element, style_for, get_image_from_uri, base_url,
-            target_collector, counter_style, state))
+            target_collector, counter_style, footnotes, state)
+
+        if child_boxes and child_boxes[0].style['float'] == 'footnote':
+            footnote = child_boxes[0]
+            footnote.style['float'] = 'none'
+            footnotes.append(footnote)
+            call_style = style_for(element, 'footnote-call')
+            footnote_call = make_box(
+                f'{element.tag}::footnote-call', call_style, [], element)
+            footnote_call.children = content_to_boxes(
+                call_style, footnote_call, quote_depth, counter_values,
+                get_image_from_uri, target_collector, counter_style)
+            footnote_call.footnote = footnote
+            child_boxes = [footnote_call]
+
+        children.extend(child_boxes)
         text = child_element.tail
         if text:
             text_box = boxes.TextBox.anonymous_from(box, text)
@@ -175,6 +212,7 @@ def element_to_box(element, style_for, get_image_from_uri, base_url,
                 children[-1].text += text_box.text
             else:
                 children.append(text_box)
+
     children.extend(before_after_to_box(
         element, 'after', state, style_for, get_image_from_uri,
         target_collector, counter_style))
@@ -186,9 +224,10 @@ def element_to_box(element, style_for, get_image_from_uri, base_url,
             counter_values.pop(name)
 
     box.children = children
-    # calculate string-set and bookmark-label
+    process_whitespace(box)
     set_content_lists(
         element, box, style, counter_values, target_collector, counter_style)
+    process_text_transform(box)
 
     if marker_boxes and len(box.children) == 1:
         # See https://www.w3.org/TR/css-lists-3/#list-style-position-outside
@@ -205,6 +244,16 @@ def element_to_box(element, style_for, get_image_from_uri, base_url,
         if style['list_style_position'] == 'outside':
             box.children.append(boxes.TextBox.anonymous_from(box, '​'))
 
+    if style['float'] == 'footnote':
+        counter_values['footnote'][-1] += 1
+        marker_style = style_for(element, 'footnote-marker')
+        marker = make_box(
+            f'{element.tag}::footnote-marker', marker_style, [], element)
+        marker.children = content_to_boxes(
+            marker_style, box, quote_depth, counter_values, get_image_from_uri,
+            target_collector, counter_style)
+        box.children.insert(0, marker)
+
     # Specific handling for the element. (eg. replaced element)
     return html.handle_element(element, box, get_image_from_uri, base_url)
 
@@ -222,10 +271,11 @@ def before_after_to_box(element, pseudo_type, state, style_for,
     # `display` differ from the computer value? It's at least wrong for
     # `content` where 'normal' computes as 'inhibit' for pseudo elements.
     display = style['display']
+    if display == ('none',):
+        return []
     content = style['content']
-    if 'none' in (display, content) or content in ('normal', 'inhibit'):
+    if content in ('normal', 'inhibit', 'none'):
         return []
-
     box = make_box(f'{element.tag}::{pseudo_type}', style, [], element)
 
     quote_depth, counter_values, _counter_scopes = state
@@ -233,7 +283,7 @@ def before_after_to_box(element, pseudo_type, state, style_for,
 
     children = []
 
-    if display == 'list-item':
+    if 'list-item' in display:
         marker_boxes = list(marker_to_box(
             element, state, style, style_for, get_image_from_uri,
             target_collector, counter_style))
@@ -274,7 +324,7 @@ def marker_to_box(element, state, parent_style, style_for, get_image_from_uri,
 
     box = make_box(f'{element.tag}::marker', style, children, element)
 
-    if style['display'] == 'none':
+    if style['display'] == ('none',):
         return
 
     image_type, image = style['list_style_image']
@@ -388,7 +438,6 @@ def compute_content_list(content_list, parent_box, counter_values, css_token,
         # in @page context. Obsoletes the parse_again function's deepcopy.
         # TODO: Is propbably superfluous in_page_context.
         parent_box.cached_counter_values = copy.deepcopy(counter_values)
-
     for type_, value in content_list:
         if type_ == 'string':
             add_text(value)
@@ -520,13 +569,14 @@ def compute_content_list(content_list, parent_box, counter_values, css_token,
                 continue
             new_box = new_box.deepcopy()
             new_box.style['position'] = 'static'
-            for child in new_box.descendants():
-                if child.style['content'] in ('normal', 'none'):
-                    continue
-                child.children = content_to_boxes(
-                    child.style, child, quote_depth, counter_values,
-                    get_image_from_uri, target_collector, counter_style,
-                    context=context, page=page)
+            if isinstance(new_box, boxes.ParentBox):
+                for child in new_box.descendants():
+                    if child.style['content'] in ('normal', 'none'):
+                        continue
+                    child.children = content_to_boxes(
+                        child.style, child, quote_depth, counter_values,
+                        get_image_from_uri, target_collector, counter_style,
+                        context=context, page=page)
             content_boxes.append(new_box)
         elif type_ == 'leader()':
             text_box = boxes.TextBox.anonymous_from(parent_box, value[1])
@@ -644,12 +694,8 @@ def compute_bookmark_label(element, box, content_list, counter_values,
     box_list = compute_content_list(
         content_list, box, counter_values, css_token, parse_again,
         target_collector, counter_style, element=element)
-
-    if box_list is None:
-        box.bookmark_label = ''
-    else:
-        box.bookmark_label = ''.join(
-            box.text for box in box_list if isinstance(box, boxes.TextBox))
+    if box_list:
+        box.bookmark_label = ''.join(box_text(box) for box in box_list)
 
 
 def set_content_lists(element, box, style, counter_values, target_collector,
@@ -700,7 +746,7 @@ def update_counters(state, style):
         # there was no counter-increment declaration for this element.
         # (Or the winning value was 'initial'.)
         # http://dev.w3.org/csswg/css3-lists/#declaring-a-list-item
-        if style['display'] == 'list-item':
+        if 'list-item' in style['display']:
             counter_increment = [('list-item', 1)]
         else:
             counter_increment = []
@@ -917,10 +963,10 @@ def wrap_table(box, children):
     footer = None
     for group in row_groups:
         display = group.style['display']
-        if display == 'table-header-group' and header is None:
+        if display == ('table-header-group',) and header is None:
             group.is_header = True
             header = group
-        elif display == 'table-footer-group' and footer is None:
+        elif display == ('table-footer-group',) and footer is None:
             group.is_footer = True
             footer = group
         else:
@@ -1178,16 +1224,7 @@ def flex_children(box, children):
                 # https://www.w3.org/TR/css-flexbox-1/#flex-items
                 continue
             if isinstance(child, boxes.InlineLevelBox):
-                # TODO: Only create block boxes for text runs, not for other
-                # inline level boxes. This is false but currently needed
-                # because block_level_width and block_level_layout are called
-                # in layout.flex.
-                if isinstance(child, boxes.ParentBox):
-                    anonymous = boxes.BlockBox.anonymous_from(
-                        box, child.children)
-                    anonymous.style = child.style
-                else:
-                    anonymous = boxes.BlockBox.anonymous_from(box, [child])
+                anonymous = boxes.BlockBox.anonymous_from(box, [child])
                 anonymous.is_flex_item = True
                 flex_children.append(anonymous)
             else:
@@ -1254,6 +1291,26 @@ def process_whitespace(box, following_collapsible_space=False):
     return following_collapsible_space
 
 
+def process_text_transform(box):
+    if isinstance(box, boxes.TextBox):
+        text_transform = box.style['text_transform']
+        if text_transform != 'none':
+            box.text = {
+                'uppercase': lambda text: text.upper(),
+                'lowercase': lambda text: text.lower(),
+                # Python’s unicode.captitalize is not the same.
+                'capitalize': lambda text: text.title(),
+                'full-width': lambda text: text.translate(ASCII_TO_WIDE),
+            }[text_transform](box.text)
+        if box.style['hyphens'] == 'none':
+            box.text = box.text.replace('\u00AD', '')  # U+00AD is soft hyphen
+
+    if isinstance(box, boxes.ParentBox) and not box.is_running():
+        for child in box.children:
+            if isinstance(child, (boxes.TextBox, boxes.InlineBox)):
+                process_text_transform(child)
+
+
 def inline_in_block(box):
     """Build the structure of lines inside blocks and return a new box tree.
 
@@ -1332,7 +1389,7 @@ def inline_in_block(box):
         if new_line_children and child_box.is_absolutely_positioned():
             new_line_children.append(child_box)
         elif isinstance(child_box, boxes.InlineLevelBox) or (
-                new_line_children and child_box.is_floated()):
+                new_line_children and not child_box.is_in_normal_flow()):
             # Do not append white space at the start of a line:
             # It would be removed during layout.
             if new_line_children or not (
@@ -1489,12 +1546,12 @@ def _inner_block_in_inline(box, skip_stack=None):
     if is_start:
         skip = 0
     else:
-        skip, skip_stack = skip_stack
+        (skip, skip_stack), = skip_stack.items()
 
     for i, child in enumerate(box.children[skip:]):
         index = i + skip
-        if isinstance(child, boxes.BlockLevelBox) and \
-                child.is_in_normal_flow():
+        if (isinstance(child, boxes.BlockLevelBox) and
+                child.is_in_normal_flow()):
             assert skip_stack is None  # Should not skip here
             block_level_box = child
             index += 1  # Resume *after* the block
@@ -1511,7 +1568,7 @@ def _inner_block_in_inline(box, skip_stack=None):
                 changed = True
             new_children.append(new_child)
         if block_level_box is not None:
-            resume_at = (index, resume_at)
+            resume_at = {index: resume_at}
             box = box.copy_with_children(new_children)
             break
     else:
