@@ -14,7 +14,6 @@
 import re
 from pkgutil import get_data
 from urllib.parse import urljoin
-from xml.etree.ElementTree import tostring
 
 from . import CSS
 from .css import get_child_text
@@ -107,11 +106,9 @@ def make_replaced_box(element, box, image):
     element should be.
 
     """
-    if box.style['display'] in ('block', 'list-item', 'table'):
-        type_ = boxes.BlockReplacedBox
-    else:
-        # TODO: support images with 'display: table-cell'?
-        type_ = boxes.InlineReplacedBox
+    type_ = (
+        boxes.BlockReplacedBox if 'block' in box.style['display']
+        else boxes.InlineReplacedBox)
     new_box = type_(element.tag, box.style, element, image)
     # TODO: check other attributes that need to be copied
     # TODO: find another solution
@@ -122,7 +119,9 @@ def make_replaced_box(element, box, image):
 
 @handler('img')
 def handle_img(element, box, get_image_from_uri, base_url):
-    """Handle ``<img>`` elements, return either an image or the alt-text.
+    """Handle ``<img>`` elements.
+
+    Return either an image or the alt-text.
 
     See: http://www.w3.org/TR/html5/embedded-content-1.html#the-img-element
 
@@ -255,19 +254,16 @@ def handle_a(element, box, _get_image_from_uri, base_url):
 
 @handler('{http://www.w3.org/2000/svg}svg')
 def handle_svg(element, box, get_image_from_uri, base_url):
-    """Handle ``<svg>`` elements, return either an image or the fallback
-    content.
+    """Handle ``<svg>`` elements.
+
+    Return either an image or the fallback content.
 
     """
     # TODO: handle href base for inline svg tags
-    if '{http://www.w3.org/2000/xmlns/}xmlns' in element.attrib:
-        # Remove default namespace to avoid xmlns being used as prefix
-        del element.attrib['{http://www.w3.org/2000/xmlns/}xmlns']
     url_fetcher = get_image_from_uri.keywords['url_fetcher']
     context = get_image_from_uri.keywords['context']
     try:
-        string = tostring(element, 'utf-8')
-        image = SVGImage(string, base_url, url_fetcher, context)
+        image = SVGImage(element, base_url, url_fetcher, context)
     except Exception as exception:
         LOGGER.error('Failed to load inline SVG: %s', exception)
         return []
