@@ -9,11 +9,12 @@
 import math
 from io import BytesIO
 from itertools import cycle
+from xml.etree import ElementTree
 
 import pydyf
 from PIL import Image
 
-from .layout.percentages import percentage
+from .layout.percent import percentage
 from .logger import LOGGER
 from .svg import SVG
 from .urls import URLFetchingError, fetch
@@ -34,7 +35,8 @@ class ImageLoadingError(ValueError):
 
 
 class RasterImage:
-    def __init__(self, pillow_image, optimize_size):
+    def __init__(self, pillow_image, image_id, optimize_size):
+        pillow_image.id = image_id
         self._pillow_image = pillow_image
         self._optimize_size = optimize_size
         self._intrinsic_width = pillow_image.width
@@ -61,8 +63,8 @@ class RasterImage:
 
 
 class SVGImage:
-    def __init__(self, string, base_url, url_fetcher, context):
-        self._svg = SVG(string, base_url)
+    def __init__(self, tree, base_url, url_fetcher, context):
+        self._svg = SVG(tree, base_url)
         self._base_url = base_url
         self._url_fetcher = url_fetcher
         self._context = context
@@ -81,6 +83,8 @@ class SVGImage:
                 ratio = None
         elif width and height:
             ratio = width / height
+        else:
+            ratio = 1
         return width, height, ratio
 
     def draw(self, stream, concrete_width, concrete_height, image_rendering):
@@ -92,10 +96,8 @@ class SVGImage:
 def get_image_from_uri(cache, url_fetcher, optimize_size, url,
                        forced_mime_type=None, context=None):
     """Get an Image instance from an image URI."""
-    missing = object()
-    image = cache.get(url, missing)
-    if image is not missing:
-        return image
+    if url in cache:
+        return cache[url]
 
     try:
         with fetch(url_fetcher, url) as result:
@@ -110,7 +112,8 @@ def get_image_from_uri(cache, url_fetcher, optimize_size, url,
             # Try to rely on given mimetype for SVG
             if mime_type == 'image/svg+xml':
                 try:
-                    image = SVGImage(string, url, url_fetcher, context)
+                    tree = ElementTree.fromstring(string)
+                    image = SVGImage(tree, url, url_fetcher, context)
                 except Exception as svg_exception:
                     svg_exceptions.append(svg_exception)
             # Try pillow for raster images, or for failing SVG
@@ -124,15 +127,16 @@ def get_image_from_uri(cache, url_fetcher, optimize_size, url,
                             svg_exceptions[0])
                     try:
                         # Last chance, try SVG
-                        image = SVGImage(string, url, url_fetcher, context)
+                        tree = ElementTree.fromstring(string)
+                        image = SVGImage(tree, url, url_fetcher, context)
                     except Exception:
                         # Tried Pillow then SVGImage for a raster, abort
                         raise ImageLoadingError.from_exception(
                             raster_exception)
                 else:
                     # Store image id to enable cache in Stream.add_image
-                    pillow_image.id = len(cache)
-                    image = RasterImage(pillow_image, optimize_size)
+                    image_id = hash(url)
+                    image = RasterImage(pillow_image, image_id, optimize_size)
 
     except (URLFetchingError, ImageLoadingError) as exception:
         LOGGER.error('Failed to load image at %r: %s', url, exception)
