@@ -11,19 +11,18 @@ from math import log10
 
 from ..css.properties import Dimension
 from ..formatting_structure import boxes
-from .percentages import resolve_one_percentage, resolve_percentages
+from .percent import resolve_one_percentage, resolve_percentages
 from .preferred import max_content_width, min_content_width
-from .tables import find_in_flow_baseline
+from .table import find_in_flow_baseline
 
 
 class FlexLine(list):
     pass
 
 
-def flex_layout(context, box, max_position_y, skip_stack, containing_block,
+def flex_layout(context, box, bottom_space, skip_stack, containing_block,
                 page_is_empty, absolute_boxes, fixed_boxes):
-    # Avoid a circular import
-    from . import blocks, preferred
+    from . import block, preferred
 
     context.create_block_formatting_context()
     resume_at = None
@@ -50,7 +49,7 @@ def flex_layout(context, box, max_position_y, skip_stack, containing_block,
                 box.padding_left - box.padding_right -
                 box.border_left_width - box.border_right_width)
         else:
-            main_space = max_position_y - box.position_y
+            main_space = context.page_bottom - bottom_space - box.position_y
             if containing_block.height != 'auto':
                 if isinstance(containing_block.height, Dimension):
                     assert containing_block.height.unit == 'px'
@@ -67,7 +66,8 @@ def flex_layout(context, box, max_position_y, skip_stack, containing_block,
         available_cross_space = getattr(box, cross)
     else:
         if cross == 'height':
-            main_space = max_position_y - box.content_box_y()
+            main_space = (
+                context.page_bottom - bottom_space - box.content_box_y())
             if containing_block.height != 'auto':
                 if isinstance(containing_block.height, Dimension):
                     assert containing_block.height.unit == 'px'
@@ -101,17 +101,19 @@ def flex_layout(context, box, max_position_y, skip_stack, containing_block,
     if parent_box.margin_right == 'auto':
         box.margin_right = parent_box.margin_right = 0
     if isinstance(parent_box, boxes.FlexBox):
-        blocks.block_level_width(parent_box, containing_block)
+        block.block_level_width(parent_box, containing_block)
     else:
         parent_box.width = preferred.flex_max_content_width(
             context, parent_box)
     original_skip_stack = skip_stack
     if skip_stack is not None:
+        # TODO: handle multiple skip stacks
+        (skip, skip_stack), = skip_stack.items()
         if box.style['flex_direction'].endswith('-reverse'):
-            children = children[:skip_stack[0] + 1]
+            children = children[:skip + 1]
         else:
-            children = children[skip_stack[0]:]
-        skip_stack = skip_stack[1]
+            children = children[skip:]
+        skip_stack = skip_stack
     else:
         skip_stack = None
     child_skip_stack = skip_stack
@@ -152,8 +154,8 @@ def flex_layout(context, box, max_position_y, skip_stack, containing_block,
             new_child.style['height'] = 'auto'
             new_child.style['min_height'] = Dimension(0, 'px')
             new_child.style['max_height'] = Dimension(float('inf'), 'px')
-            new_child = blocks.block_level_layout(
-                context, new_child, float('inf'), child_skip_stack,
+            new_child = block.block_level_layout(
+                context, new_child, -float('inf'), child_skip_stack,
                 parent_box, page_is_empty, [], [], [], False)[0]
             content_size = new_child.height
             child.min_height = min(specified_size, content_size)
@@ -211,8 +213,8 @@ def flex_layout(context, box, max_position_y, skip_stack, containing_block,
                     else:
                         new_child = child.copy()
                     new_child.width = float('inf')
-                    new_child = blocks.block_level_layout(
-                        context, new_child, float('inf'), child_skip_stack,
+                    new_child = block.block_level_layout(
+                        context, new_child, -float('inf'), child_skip_stack,
                         parent_box, page_is_empty, absolute_boxes, fixed_boxes,
                         adjoining_margins=[], discard=False)[0]
                     child.flex_base_size = new_child.margin_height()
@@ -226,8 +228,8 @@ def flex_layout(context, box, max_position_y, skip_stack, containing_block,
                     else:
                         new_child = child.copy()
                     new_child.width = 0
-                    new_child = blocks.block_level_layout(
-                        context, new_child, float('inf'), child_skip_stack,
+                    new_child = block.block_level_layout(
+                        context, new_child, -float('inf'), child_skip_stack,
                         parent_box, page_is_empty, absolute_boxes, fixed_boxes,
                         adjoining_margins=[], discard=False)[0]
                     child.flex_base_size = new_child.margin_height()
@@ -246,7 +248,7 @@ def flex_layout(context, box, max_position_y, skip_stack, containing_block,
     # Step 4
     # TODO: the whole step has to be fixed
     if axis == 'width':
-        blocks.block_level_width(box, containing_block)
+        block.block_level_width(box, containing_block)
     else:
         if box.style['height'] != 'auto':
             box.height = box.style['height'].value
@@ -458,10 +460,10 @@ def flex_layout(context, box, max_position_y, skip_stack, containing_block,
                 child_copy = child.copy_with_children(child.children)
             else:
                 child_copy = child.copy()
-            blocks.block_level_width(child_copy, parent_box)
+            block.block_level_width(child_copy, parent_box)
             new_child, _, _, adjoining_margins, _ = (
-                blocks.block_level_layout_switch(
-                    context, child_copy, float('inf'), child_skip_stack,
+                block.block_level_layout_switch(
+                    context, child_copy, -float('inf'), child_skip_stack,
                     parent_box, page_is_empty, absolute_boxes, fixed_boxes,
                     adjoining_margins=[], discard=False))
 
@@ -471,8 +473,7 @@ def flex_layout(context, box, max_position_y, skip_stack, containing_block,
                 # As flex items margins never collapse (with other flex items
                 # or with the flex container), we can add the adjoining margins
                 # to the child bottom margin.
-                child.margin_bottom += blocks.collapse_margin(
-                    adjoining_margins)
+                child.margin_bottom += block.collapse_margin(adjoining_margins)
             else:
                 child.width = min_content_width(context, child, outer=False)
 
@@ -839,24 +840,26 @@ def flex_layout(context, box, max_position_y, skip_stack, containing_block,
     for line in flex_lines:
         for i, child in line:
             if child.is_flex_item:
-                new_child, child_resume_at = blocks.block_level_layout_switch(
-                    context, child, max_position_y, child_skip_stack, box,
+                new_child, child_resume_at = block.block_level_layout_switch(
+                    context, child, bottom_space, child_skip_stack, box,
                     page_is_empty, absolute_boxes, fixed_boxes,
                     adjoining_margins=[], discard=False)[:2]
                 if new_child is None:
                     if resume_at:
                         index, = resume_at
                         if index:
-                            resume_at = (index + i - 1, None)
+                            resume_at = {index + i - 1: None}
                 else:
                     box.children.append(new_child)
                     if child_resume_at is not None:
                         if original_skip_stack:
-                            first_level_skip = original_skip_stack[0]
+                            # TODO: handle multiple skip stacks
+                            first_level_skip, = original_skip_stack
                         else:
                             first_level_skip = 0
                         if resume_at:
-                            first_level_skip += tuple(resume_at)[0]
+                            index, = resume_at
+                            first_level_skip += index
                         resume_at = {first_level_skip + i: child_resume_at}
                 if resume_at:
                     break
