@@ -198,19 +198,16 @@ def inline_min_content_width(context, box, outer=True, skip_stack=None,
     widths = inline_line_widths(
         context, box, outer, is_line_start, minimum=True,
         skip_stack=skip_stack, first_line=first_line)
-
-    if first_line:
-        widths = [next(widths)]
-    else:
-        widths = list(widths)
-        widths[-1] -= trailing_whitespace_size(context, box)
-    return adjust(box, outer, max(widths))
+    width = next(widths) if first_line else max(widths)
+    return adjust(box, outer, width)
 
 
 def inline_max_content_width(context, box, outer=True, is_line_start=False):
     """Return the max-content width for an ``InlineBox``."""
     widths = list(
         inline_line_widths(context, box, outer, is_line_start, minimum=False))
+    # Remove trailing space, as split_first_line keeps trailing spaces when
+    # max_width is not set.
     widths[-1] -= trailing_whitespace_size(context, box)
     return adjust(box, outer, max(widths))
 
@@ -268,7 +265,8 @@ def inline_line_widths(context, box, outer, is_line_start, minimum,
     if skip_stack is None:
         skip = 0
     else:
-        skip, skip_stack = skip_stack
+        # TODO: handle multiple skip stacks
+        (skip, skip_stack), = skip_stack.items()
     for child in box.children[skip:]:
         if child.is_absolutely_positioned():
             continue  # Skip
@@ -292,7 +290,7 @@ def inline_line_widths(context, box, outer, is_line_start, minimum,
             if skip_stack is None:
                 skip = 0
             else:
-                skip, skip_stack = skip_stack
+                (skip, skip_stack), = skip_stack.items()
                 assert skip_stack is None
             child_text = child.text[(skip or 0):]
             if is_line_start and space_collapse:
@@ -302,18 +300,18 @@ def inline_line_widths(context, box, outer, is_line_start, minimum,
             else:
                 max_width = 0 if minimum else None
                 lines = []
-                resume_at = new_resume_at = 0
-                while new_resume_at is not None:
-                    resume_at += new_resume_at
-                    _, _, new_resume_at, width, _, _ = (
+                resume_index = new_resume_index = 0
+                while new_resume_index is not None:
+                    resume_index += new_resume_index
+                    _, _, new_resume_index, width, _, _ = (
                         split_first_line(
-                            child_text[resume_at:], child.style, context,
+                            child_text[resume_index:], child.style, context,
                             max_width, child.justification_spacing,
                             minimum=True))
                     lines.append(width)
                     if first_line:
                         break
-                if first_line and new_resume_at:
+                if first_line and new_resume_index:
                     current_line += lines[0]
                     break
         else:
@@ -373,8 +371,7 @@ def table_and_columns_preferred_widths(context, box, outer=True):
     http://dbaron.org/css/intrinsic/
 
     """
-    # Avoid a circular import
-    from .tables import distribute_excess_width
+    from .table import distribute_excess_width
 
     table = box.get_wrapped_table()
     result = context.tables.get(table)
@@ -731,7 +728,7 @@ def flex_max_content_width(context, box, outer=True):
 
 def trailing_whitespace_size(context, box):
     """Return the size of the trailing whitespace of ``box``."""
-    from .inlines import split_first_line, split_text_box
+    from .inline import split_first_line, split_text_box
 
     while isinstance(box, (boxes.InlineBox, boxes.LineBox)):
         if not box.children:
