@@ -157,20 +157,28 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
                     cell.remove_decoration(start=True, end=False)
                     if table.style['border_collapse'] == 'collapse':
                         table.skip_cell_border_top = True
+
+                # First try to render content as if there was already something
+                # on the page to avoid hitting block_level_layout’s TODO. Then
+                # force to render something if the page is actually empty, or
+                # just draw an empty cell otherwise. See
+                # test_table_break_children_margin.
                 new_cell, cell_resume_at, _, _, _ = block_container_layout(
                     context, cell, bottom_space, cell_skip_stack,
-                    page_is_empty=False, absolute_boxes=absolute_boxes,
+                    page_is_empty=page_is_empty, absolute_boxes=absolute_boxes,
                     fixed_boxes=fixed_boxes, adjoining_margins=None,
                     discard=False)
                 if new_cell is None:
                     cell = cell.copy_with_children([])
                     cell, _, _, _, _ = block_container_layout(
                         context, cell, bottom_space, cell_skip_stack,
-                        page_is_empty=True, absolute_boxes=[], fixed_boxes=[],
-                        adjoining_margins=None, discard=False)
+                        page_is_empty=True, absolute_boxes=[],
+                        fixed_boxes=[], adjoining_margins=None,
+                        discard=False)
                     cell_resume_at = {0: None}
                 else:
                     cell = new_cell
+
                 cell.remove_decoration(
                     start=cell_skip_stack is not None,
                     end=cell_resume_at is not None)
@@ -186,6 +194,11 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
                     cell.height = max(cell.height, cell.computed_height)
                 new_row_children.append(cell)
 
+            if resume_at and not page_is_empty:
+                if row.style['break_inside'] in ('avoid', 'avoid-page'):
+                    resume_at = {index_row: {}}
+                    break
+
             row = row.copy_with_children(new_row_children)
 
             # Table height algorithm
