@@ -12,6 +12,7 @@ from xml.etree import ElementTree
 
 from cssselect2 import ElementWrapper
 
+from ..urls import get_url_attribute
 from .bounding_box import bounding_box, is_valid_bounding_box
 from .css import parse_declarations, parse_stylesheets
 from .defs import (
@@ -175,9 +176,12 @@ class Node:
             return tuple(
                 float(number) for number in normalize(viewbox).split())
 
-    def get_href(self):
+    def get_href(self, base_url):
         """Get the href attribute, with or without a namespace."""
-        return self.get('{http://www.w3.org/1999/xlink}href', self.get('href'))
+        for attr_name in ('{http://www.w3.org/1999/xlink}href', 'href'):
+            url = get_url_attribute(self, attr_name, base_url)
+            if url:
+                return url
 
     @staticmethod
     def process_whitespace(string, preserve):
@@ -279,8 +283,7 @@ class Node:
 class SVG:
     """An SVG document."""
 
-    def __init__(self, bytestring_svg, url):
-        tree = ElementTree.fromstring(bytestring_svg)
+    def __init__(self, tree, url):
         wrapper = ElementWrapper.from_xml_root(tree)
         style = parse_stylesheets(wrapper, url)
         self.tree = Node(wrapper, style)
@@ -294,6 +297,8 @@ class SVG:
         self.patterns = {}
         self.paths = {}
 
+        self.use_cache = {}
+
         self.cursor_position = [0, 0]
         self.cursor_d_position = [0, 0]
         self.text_path_width = 0
@@ -407,8 +412,8 @@ class SVG:
                 self.stream.transform(*(old_ctm @ new_ctm.invert).values)
 
         # Manage display and visibility
-        display = node.get('display', 'inline') != 'none'
-        visible = display and (node.get('visibility', 'visible') != 'hidden')
+        display = node.get('display') != 'none'
+        visible = display and (node.get('visibility') != 'hidden')
 
         # Draw node
         if visible and node.tag in TAGS:
