@@ -22,7 +22,7 @@ class TextBox:
 def text(svg, node, font_size):
     """Draw text node."""
     from ..css.properties import INITIAL_VALUES
-    from ..draw import draw_first_line
+    from ..draw import draw_emojis, draw_first_line
     from ..text.line_break import split_first_line
 
     # TODO: use real computed values
@@ -75,7 +75,7 @@ def text(svg, node, font_size):
     letter_spacing = svg.length(node.get('letter-spacing'), font_size)
     text_anchor = node.get('text-anchor')
     # TODO: use real values
-    ascent, descent = 100, 20
+    ascent, descent = font_size * .8, font_size * .2
     if text_anchor == 'middle':
         x_align = - (width / 2. + x_bearing)
         if letter_spacing and node.text:
@@ -121,6 +121,7 @@ def text(svg, node, font_size):
         return
 
     svg.stream.begin_text()
+    emoji_lines = []
 
     # Draw letters
     for i, ((x, y, dx, dy, r), letter) in enumerate(letters_positions):
@@ -160,10 +161,14 @@ def text(svg, node, font_size):
 
         layout.reactivate(style)
         svg.fill_stroke(node, font_size, text=True)
-        draw_first_line(
+        emojis = draw_first_line(
             svg.stream, TextBox(layout, style), 'none', 'none',
             x_position, y_position)
+        emoji_lines.append((font_size, x, y, emojis))
         svg.stream.pop_state()
         svg.cursor_position = cursor_position
 
     svg.stream.end_text()
+
+    for font_size, x, y, emojis in emoji_lines:
+        draw_emojis(svg.stream, font_size, x, y, emojis)
