@@ -6,6 +6,8 @@
 
 """
 
+import os
+
 import cffi
 
 ffi = cffi.FFI()
@@ -15,10 +17,20 @@ ffi.cdef('''
     typedef ... hb_font_t;
     typedef ... hb_face_t;
     typedef ... hb_blob_t;
+    typedef uint32_t hb_codepoint_t;
     hb_face_t * hb_font_get_face (hb_font_t *font);
     hb_blob_t * hb_face_reference_blob (hb_face_t *face);
     unsigned int hb_face_get_index (const hb_face_t *face);
+    unsigned int hb_face_get_upem (const hb_face_t *face);
     const char * hb_blob_get_data (hb_blob_t *blob, unsigned int *length);
+    bool hb_ot_color_has_png (hb_face_t *face);
+    hb_blob_t * hb_ot_color_glyph_reference_png (
+        hb_font_t *font, hb_codepoint_t glyph);
+    bool hb_ot_color_has_svg (hb_face_t *face);
+    hb_blob_t * hb_ot_color_glyph_reference_svg (
+        hb_face_t *face, hb_codepoint_t glyph);
+    void hb_blob_destroy (hb_blob_t *blob);
+
 
     // Pango
 
@@ -272,6 +284,7 @@ ffi.cdef('''
         PangoAttrList *list, PangoAttribute *attr);
     PangoAttribute * pango_attr_font_features_new (const gchar *features);
     PangoAttribute * pango_attr_letter_spacing_new (int letter_spacing);
+    PangoAttribute * pango_attr_insert_hyphens_new (gboolean insert_hyphens);
     void pango_attribute_destroy (PangoAttribute *attr);
 
     PangoTabArray * pango_tab_array_new_with_positions (
@@ -377,21 +390,32 @@ def _dlopen(ffi, *names):
     return ffi.dlopen(names[0])  # pragma: no cover
 
 
+if hasattr(os, 'add_dll_directory'):
+    dll_directories = os.getenv(
+        'WEASYPRINT_DLL_DIRECTORIES',
+        'C:\\Program Files\\GTK3-Runtime Win64\\bin').split(';')
+    for dll_directory in dll_directories:
+        try:
+            os.add_dll_directory(dll_directory)
+        except (OSError, FileNotFoundError):
+            pass
+
 gobject = _dlopen(
     ffi, 'gobject-2.0-0', 'gobject-2.0', 'libgobject-2.0-0',
-    'libgobject-2.0.so.0', 'libgobject-2.0.dylib')
+    'libgobject-2.0.so.0', 'libgobject-2.0.dylib',  'libgobject-2.0-0.dll')
 pango = _dlopen(
     ffi, 'pango-1.0-0', 'pango-1.0', 'libpango-1.0-0', 'libpango-1.0.so.0',
-    'libpango-1.0.dylib')
+    'libpango-1.0.dylib', 'libpango-1.0-0.dll')
 harfbuzz = _dlopen(
     ffi, 'harfbuzz', 'harfbuzz-0.0', 'libharfbuzz-0',
-    'libharfbuzz.so.0', 'libharfbuzz.so.0', 'libharfbuzz.0.dylib')
+    'libharfbuzz.so.0', 'libharfbuzz.so.0', 'libharfbuzz.0.dylib',
+    'libharfbuzz-0.dll')
 fontconfig = _dlopen(
-    ffi, 'fontconfig-1', 'fontconfig', 'libfontconfig', 'libfontconfig-1.dll',
-    'libfontconfig.so.1', 'libfontconfig-1.dylib')
+    ffi, 'fontconfig-1', 'fontconfig', 'libfontconfig', 'libfontconfig.so.1',
+    'libfontconfig-1.dylib', 'libfontconfig-1.dll')
 pangoft2 = _dlopen(
     ffi, 'pangoft2-1.0-0', 'pangoft2-1.0', 'libpangoft2-1.0-0',
-    'libpangoft2-1.0.so.0', 'libpangoft2-1.0.dylib')
+    'libpangoft2-1.0.so.0', 'libpangoft2-1.0.dylib', 'libpangoft2-1.0-0.dll')
 
 gobject.g_type_init()
 
