@@ -6,6 +6,7 @@
 
 """
 
+import hashlib
 import io
 import os
 import pathlib
@@ -135,6 +136,7 @@ class FontConfiguration:
                 # Back to default.
                 self._tempdir = None
         self._filenames = []
+        self._woff_cache = {}
 
     def add_font_face(self, rule_descriptors, url_fetcher):
         if self.font_map is None:
@@ -194,21 +196,35 @@ class FontConfiguration:
                             font = result['string']
                         else:
                             font = result['file_obj'].read()
-                    if font[:3] == b'wOF':
-                        out = io.BytesIO()
-                        if font[3:4] == b'F':
-                            # woff font
-                            ttfont = TTFont(io.BytesIO(font))
-                            ttfont.flavor = ttfont.flavorData = None
-                            ttfont.save(out)
-                        elif font[3:4] == b'2':
-                            # woff2 font
-                            woff2.decompress(io.BytesIO(font), out)
-                        font = out.getvalue()
                 except Exception as exc:
                     LOGGER.debug(
                         'Failed to load font at %r (%s)', url, exc)
                     continue
+                try:
+                    font_is_woff = font[:3] == b'wOF'
+                    if font_is_woff:
+                        font_hasher = hashlib.sha256()
+                        font_hasher.update(font)
+                        font_digest = font_hasher.hexdigest()
+                        decoded_woff_path = self._woff_cache.get(font_digest)
+                        if decoded_woff_path is None:
+                            out = io.BytesIO()
+                            woff_version_byte = font[3:4]
+                            if woff_version_byte == b'F':
+                                # woff font
+                                ttfont = TTFont(io.BytesIO(font))
+                                ttfont.flavor = ttfont.flavorData = None
+                                ttfont.save(out)
+                            elif woff_version_byte == b'2':
+                                # woff2 font
+                                woff2.decompress(io.BytesIO(font), out)
+                            font = out.getvalue()
+                        else:
+                            font = pathlib.Path(decoded_woff_path).read_bytes()
+                except Exception as exc:
+                    LOGGER.debug(
+                        'Failed to handle woff font at %r (%s)', url, exc)
+                    continue
                 features = {
                     rules[0][0].replace('-', '_'): rules[0][1] for rules in
                     rule_descriptors.get('font_variant', [])}
@@ -224,6 +240,8 @@ class FontConfiguration:
                 fd.write(font)
                 fd.close()
                 self._filenames.append(font_filename)
+                if font_is_woff and font_digest not in self._woff_cache:
+                    self._woff_cache[font_digest] = font_filename
                 fontconfig_style = FONTCONFIG_STYLE[
                     rule_descriptors.get('font_style', 'normal')]
                 fontconfig_weight = FONTCONFIG_WEIGHT[
