@@ -212,9 +212,10 @@ class Layout:
         if letter_spacing == 'normal':
             letter_spacing = 0
 
-        if text and (word_spacing != 0 or letter_spacing != 0):
-            letter_spacing = units_from_double(letter_spacing)
-            space_spacing = units_from_double(word_spacing) + letter_spacing
+        word_breaking = (
+            self.style['overflow_wrap'] in ('anywhere', 'break-word'))
+
+        if self.text and (word_spacing or letter_spacing or word_breaking):
             attr_list = pango.pango_layout_get_attributes(self.layout)
             if not attr_list:
                 # TODO: list should be freed
@@ -226,11 +227,23 @@ class Layout:
                 attr.start_index, attr.end_index = start, end
                 pango.pango_attr_list_change(attr_list, attr)
 
-            add_attr(0, len(bytestring), letter_spacing)
-            position = bytestring.find(b' ')
-            while position != -1:
-                add_attr(position, position + 1, space_spacing)
-                position = bytestring.find(b' ', position + 1)
+            if letter_spacing:
+                letter_spacing = units_from_double(letter_spacing)
+                add_attr(0, len(bytestring), letter_spacing)
+
+            if word_spacing:
+                space_spacing = (
+                    units_from_double(word_spacing) + letter_spacing)
+                position = bytestring.find(b' ')
+                while position != -1:
+                    add_attr(position, position + 1, space_spacing)
+                    position = bytestring.find(b' ', position + 1)
+
+            if word_breaking:
+                # TODO: attributes should be freed
+                attr = pango.pango_attr_insert_hyphens_new(False)
+                attr.start_index, attr.end_index = 0, len(bytestring)
+                pango.pango_attr_list_change(attr_list, attr)
 
             pango.pango_layout_set_attributes(self.layout, attr_list)
 
@@ -532,7 +545,8 @@ def split_first_line(text, style, context, max_width, justification_spacing,
     first_line_width, _ = line_size(first_line, style)
     space = max_width - first_line_width
     # If we can break words and the first line is too long
-    if not minimum and overflow_wrap == 'break-word' and space < 0:
+    if space < 0 and (overflow_wrap == 'anywhere' or
+                      (overflow_wrap == 'break-word' and not minimum)):
         # Is it really OK to remove hyphenation for word-break ?
         hyphenated = False
         # TODO: Modify code to preserve W3C condition:
